@@ -1,6 +1,6 @@
 """
 This module defines the different types of terms. Terms are the kinds of
-objects that can appear in a quoted/asserted triple. This includes those 
+objects that can appear in a quoted/asserted triple. This includes those
 that are core to RDF:
 
 * Blank Nodes
@@ -12,7 +12,8 @@ Those that extend the RDF model into N3:
 * Formulae
 * Universal Quantifications (Variables)
 
-And those that are primarily for matching against 'Nodes' in the underlying Graph:
+And those that are primarily for matching against 'Nodes' in the
+underlying Graph:
 
 * REGEX Expressions
 * Date Ranges
@@ -22,7 +23,7 @@ And those that are primarily for matching against 'Nodes' in the underlying Grap
 
 __all__ = [
     'bind',
-    
+
     'Node',
     'Identifier',
 
@@ -32,38 +33,31 @@ __all__ = [
 
     'Variable',
     'Statement',
-    ]
+]
 
 import logging
+import warnings
 
 _LOGGER = logging.getLogger(__name__)
 
 import base64
-import sys
-import threading
 
 from urlparse import urlparse, urljoin, urldefrag
-from string import ascii_letters
-from random import choice
-from itertools import islice
 from datetime import date, time, datetime
 from isodate import parse_time, parse_date, parse_datetime
 from re import sub
 
+
 try:
     from hashlib import md5
+    assert md5
 except ImportError:
     from md5 import md5
 
-try:
-    from uuid import uuid4
-except ImportError:
-    import os
-    import random
-
-import py3compat
+from . import py3compat
 b = py3compat.b
 
+
 class Node(object):
     """
     A Node in the Graph.
@@ -72,7 +66,7 @@ class Node(object):
     __slots__ = ()
 
 
-class Identifier(Node, unicode): # we allow Identifiers to be Nodes in our Graph
+class Identifier(Node, unicode):  # allow Identifiers to be Nodes in the Graph
     """
     See http://www.w3.org/2002/07/rdf-identifer-terminology/
     regarding choice of terminology.
@@ -98,7 +92,7 @@ class URIRef(Identifier):
             if ends_in_hash:
                 if not value.endswith("#"):
                     value += "#"
-        #if normalize and value and value != normalize("NFC", value):
+        # if normalize and value and value != normalize("NFC", value):
         #    raise Error("value must be in NFC normalized form.")
         try:
             rt = unicode.__new__(cls, value)
@@ -113,12 +107,16 @@ class URIRef(Identifier):
         return "<%s>" % self
 
     def concrete(self):
+        warnings.warn("URIRef.concrete is deprecated.",
+                      category=DeprecationWarning, stacklevel=2)
         if "#" in self:
             return URIRef("/".join(self.rsplit("#", 1)))
         else:
             return self
 
     def abstract(self):
+        warnings.warn("URIRef.abstract is deprecated.",
+                      category=DeprecationWarning, stacklevel=2)
         if "#" not in self:
             scheme, netloc, path, params, query, fragment = urlparse(self)
             if path:
@@ -131,7 +129,6 @@ class URIRef(Identifier):
         else:
             return self
 
-
     def defrag(self):
         if "#" in self:
             url, frag = urldefrag(self)
@@ -145,16 +142,15 @@ class URIRef(Identifier):
     def __getnewargs__(self):
         return (unicode(self), )
 
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
     def __eq__(self, other):
         if isinstance(other, URIRef):
-            return unicode(self)==unicode(other)
+            return unicode(self) == unicode(other)
         else:
             return False
-    
+
     def __hash__(self):
         return hash(URIRef) ^ hash(unicode(self))
 
@@ -168,8 +164,7 @@ class URIRef(Identifier):
         else:
             clsName = self.__class__.__name__
 
-        return """%s(%s)""" % (clsName, super(URIRef,self).__repr__())
-        
+        return """%s(%s)""" % (clsName, super(URIRef, self).__repr__())
 
     def md5_term_hash(self):
         """a string of hex that will be the same for two URIRefs that
@@ -187,74 +182,99 @@ def _unique_id():
     # Used to read: """Create a (hopefully) unique prefix"""
     # now retained merely to leave interal API unchanged.
     # From BNode.__new__() below ...
-    # 
+    #
     # acceptable bnode value range for RDF/XML needs to be
     # something that can be serialzed as a nodeID for N3
-    # 
+    #
     # BNode identifiers must be valid NCNames" _:[A-Za-z][A-Za-z0-9]*
     # http://www.w3.org/TR/2004/REC-rdf-testcases-20040210/#nodeID
-    return "_"
+    return "N"  # ensure that id starts with a letter
 
-# Adapted from http://icodesnip.com/snippet/python/simple-universally-unique-id-uuid-or-guid
+
+# Adapted from http://icodesnip.com/snippet/python/
+# simple-universally-unique-id-uuid-or-guid
 def bnode_uuid():
     """
     Generates a uuid on behalf of Python 2.4
     """
+    import os
+    import random
     import socket
-    import time
+    from time import time
+    from binascii import hexlify
+
+    pid = [None]
+
     try:
-        preseed = os.urandom(16)
-    except NotImplementedError: 
-        preseed = ''
-    # Have doubts about this. random.seed will just hash the string
-    random.seed('%s%s%s' % (preseed, os.getpid(), time.time()))
-    del preseed
-    t = long(time.time() * 1000.0)
-    r = long(random.random()*100000000000000000L)
-    try:
-        a = socket.gethostbyname(socket.gethostname())
+        ip = socket.gethostbyname(socket.gethostname())
+        ip = long(ip.replace('.', '999').replace(':', '999'))
     except:
         # if we can't get a network address, just imagine one
-        a = random.random()*100000000000000000L
-    data = str(t) + ' ' + str(r) + ' ' + str(a)
-    data = md5(data.encode('ascii')).hexdigest()
-    return '-'.join(
-            [data[:8], data[8:12], data[12:16], data[16:20], data[20:]])
+        ip = long(random.random() * 100000000000000000L)
+
+    def _generator():
+        if os.getpid() != pid[0]:
+            # Process might have been forked (issue 200), must reseed random:
+            try:
+                preseed = long(hexlify(os.urandom(16)), 16)
+            except NotImplementedError:
+                preseed = 0
+            seed = long(str(preseed) + str(os.getpid())
+                        + str(long(time() * 1000000)) + str(ip))
+            random.seed(seed)
+            pid[0] = os.getpid()
+
+        t = long(time() * 1000.0)
+        r = long(random.random() * 100000000000000000L)
+        data = str(t) + ' ' + str(r) + ' ' + str(ip)
+        return md5(data).hexdigest()
+
+    return _generator
+
+
+def uuid4_ncname():
+    """
+    Generates UUID4-based but ncname-compliant identifiers.
+    """
+    from uuid import uuid4
+
+    def _generator():
+        return uuid4().hex
+
+    return _generator
+
 
 def _serial_number_generator():
-    while 1:
-        if sys.version_info[:2] < (2, 5):
-            yield bnode_uuid()
-        else:
-            yield uuid4()
+    import sys
+    if sys.version_info[:2] < (2, 5):
+        return bnode_uuid()
+    else:
+        return uuid4_ncname()
 
-# bNodeLock = threading.RLock()
 
 class BNode(Identifier):
     """
-    Blank Node: http://www.w3.org/TR/rdf-concepts/#section-blank-nodes    
+    Blank Node: http://www.w3.org/TR/rdf-concepts/#section-blank-nodes
 
     """
     __slots__ = ()
 
-
-    def __new__(cls, value=None, 
+    def __new__(cls, value=None,
                 _sn_gen=_serial_number_generator(), _prefix=_unique_id()):
         """
         # only store implementations should pass in a value
         """
-        if value==None:
-            # so that BNode values do not
-            # collide with ones created with a different instance of this module
-            # at some other time.
-            node_id = _sn_gen.next()
+        if value is None:
+            # so that BNode values do not collide with ones created with
+            # a different instance of this module at some other time.
+            node_id = _sn_gen()
             value = "%s%s" % (_prefix, node_id)
         else:
             # TODO: check that value falls within acceptable bnode value range
             # for RDF/XML needs to be something that can be serialzed
             # as a nodeID for N3 ??  Unless we require these
             # constraints be enforced elsewhere?
-            pass # assert is_ncname(unicode(value)), "BNode identifiers
+            pass  # assert is_ncname(unicode(value)), "BNode identifiers
                  # must be valid NCNames" _:[A-Za-z][A-Za-z0-9]*
                  # http://www.w3.org/TR/2004/REC-rdf-testcases-20040210/#nodeID
         return Identifier.__new__(cls, value)
@@ -288,10 +308,10 @@ class BNode(Identifier):
         True
         """
         if isinstance(other, BNode):
-            return unicode(self)==unicode(other)
+            return unicode(self) == unicode(other)
         else:
             return False
-    
+
     def __hash__(self):
         return hash(BNode) ^ hash(unicode(self))
 
@@ -358,8 +378,8 @@ class Literal(Identifier):
     >>> "2005" < lit2006
     True
     >>> x = Literal("2", datatype=XSD.integer)
-    >>> x
-    rdflib.term.Literal(%(u)s'2', datatype=rdflib.term.URIRef(%(u)s'http://www.w3.org/2001/XMLSchema#integer'))
+    >>> x.n3()
+    %(u)s'"2"^^<http://www.w3.org/2001/XMLSchema#integer>'
     >>> Literal(x) == x
     True
     >>> x = Literal("cake", lang="en")
@@ -370,16 +390,21 @@ class Literal(Identifier):
     """
     __doc__ = py3compat.format_doctest_out(doc)
 
-    __slots__ = ("language", "datatype", "_cmp_value")
+    if not py3compat.PY3:
+        __slots__ = (
+            "language", "datatype", "_language", "_datatype", "_cmp_value")
+    else:
+        __slots__ = ("_language", "_datatype", "_cmp_value")
 
     def __new__(cls, value, lang=None, datatype=None):
         if lang is not None and datatype is not None:
-            raise TypeError("A Literal can only have one of lang or datatype, "
-               "per http://www.w3.org/TR/rdf-concepts/#section-Graph-Literal")
+            raise TypeError(
+                "A Literal can only have one of lang or datatype, "
+                "per http://www.w3.org/TR/rdf-concepts/#section-Graph-Literal")
 
-        if isinstance(value, Literal): # create from another Literal instance
-            datatype=datatype or value.datatype
-            lang=lang or value.language
+        if isinstance(value, Literal):  # create from another Literal instance
+            datatype = datatype or value.datatype
+            lang = lang or value.language
 
         if datatype:
             lang = None
@@ -395,11 +420,19 @@ class Literal(Identifier):
             inst = unicode.__new__(cls, value)
         except UnicodeDecodeError:
             inst = unicode.__new__(cls, value, 'utf-8')
-        inst.language = lang
-        inst.datatype = datatype
+        inst._language = lang
+        inst._datatype = datatype
         inst._cmp_value = inst._toCompareValue()
         return inst
 
+    @property
+    def language(self):
+        return self._language
+
+    @property
+    def datatype(self):
+        return self._datatype
+
     def __reduce__(self):
         return (Literal, (unicode(self), self.language, self.datatype),)
 
@@ -408,8 +441,8 @@ class Literal(Identifier):
 
     def __setstate__(self, arg):
         _, d = arg
-        self.language = d["language"]
-        self.datatype = d["datatype"]
+        self._language = d["language"]
+        self._datatype = d["datatype"]
 
     @py3compat.format_doctest_out
     def __add__(self, val):
@@ -437,19 +470,19 @@ class Literal(Identifier):
         >>> from rdflib.namespace import XSD
         >>> (- Literal("1", datatype=XSD['integer']))
         -1%(L)s
-        
+
         Not working:
         #>>> (- Literal("1"))
         #Traceback (most recent call last):
         #  File "<stdin>", line 1, in <module>
         #TypeError: Not a number; rdflib.term.Literal(u'1')
-        >>> 
+        >>>
         """
 
         py = self.toPython()
         try:
             return py.__neg__()
-        except Exception, e:
+        except Exception:
             raise TypeError("Not a number; %s" % repr(self))
 
     @py3compat.format_doctest_out
@@ -462,7 +495,7 @@ class Literal(Identifier):
         >>> from rdflib.namespace import XSD
         >>> (+ Literal("-1", datatype=XSD['integer']))
         -1%(L)s
-        
+
         Not working in Python 3:
         #>>> (+ Literal("1"))
         #Traceback (most recent call last):
@@ -472,7 +505,7 @@ class Literal(Identifier):
         py = self.toPython()
         try:
             return py.__pos__()
-        except Exception, e:
+        except Exception:
             raise TypeError("Not a number; %s" % repr(self))
 
     @py3compat.format_doctest_out
@@ -483,7 +516,7 @@ class Literal(Identifier):
         >>> from rdflib.namespace import XSD
         >>> abs( Literal("-1", datatype=XSD['integer']))
         1%(L)s
-        
+
         Not working in Python 3:
         #>>> abs(Literal("1"))
         #Traceback (most recent call last):
@@ -493,7 +526,7 @@ class Literal(Identifier):
         py = self.toPython()
         try:
             return py.__abs__()
-        except Exception, e:
+        except Exception:
             raise TypeError("Not a number; %s" % repr(self))
 
     @py3compat.format_doctest_out
@@ -504,13 +537,13 @@ class Literal(Identifier):
         >>> from rdflib.namespace import XSD
         >>> ~( Literal("-1", datatype=XSD['integer']))
         0%(L)s
-        
+
         Not working:
         #>>> ~(Literal("1"))
         #Traceback (most recent call last):
         #  File "<stdin>", line 1, in <module>
         #TypeError: Not a number; rdflib.term.Literal(u'1')
-        >>> 
+        >>>
         """
         py = self.toPython()
         try:
@@ -526,12 +559,15 @@ class Literal(Identifier):
         True
         >>> %(u)s"\xfe" < Literal(%(u)s"foo")
         False
-        >>> Literal(base64.encodestring(%(u)s"\xfe".encode("utf-8")), datatype=URIRef("http://www.w3.org/2001/XMLSchema#base64Binary")) < %(u)s"foo"
+        >>> Literal(base64.encodestring(%(u)s"\xfe".encode("utf-8")),
+        ...     datatype=URIRef(
+        ...         "http://www.w3.org/2001/XMLSchema#base64Binary")
+        ...          ) < %(u)s"foo"
         False
         """
 
         if other is None:
-            return False # Nothing is less than None
+            return False  # Nothing is less than None
         try:
             return self._cmp_value < other
         except UnicodeDecodeError, ue:
@@ -541,7 +577,8 @@ class Literal(Identifier):
                 raise ue
         except TypeError:
             try:
-                # On Python 3, comparing bytes/str is a TypeError, not a UnicodeError
+                # On Python 3, comparing bytes/str is a TypeError,
+                # not a UnicodeError
                 if isinstance(self._cmp_value, py3compat.bytestype):
                     return self._cmp_value < other.encode("utf-8")
                 return unicode(self._cmp_value) < other
@@ -552,19 +589,20 @@ class Literal(Identifier):
     def __le__(self, other):
         """
         >>> from rdflib.namespace import XSD
-        >>> Literal('2007-01-01T10:00:00', datatype=XSD.dateTime) <= Literal('2007-01-01T10:00:00', datatype=XSD.dateTime)
+        >>> Literal('2007-01-01T10:00:00', datatype=XSD.dateTime
+        ...     ) <= Literal('2007-01-01T10:00:00', datatype=XSD.dateTime)
         True
         """
         if other is None:
             return False
-        if self==other:
+        if self == other:
             return True
         else:
             return self < other
 
     def __gt__(self, other):
         if other is None:
-            return True # Everything is greater than None
+            return True  # Everything is greater than None
         try:
             return self._cmp_value > other
         except UnicodeDecodeError, ue:
@@ -574,7 +612,8 @@ class Literal(Identifier):
                 raise ue
         except TypeError:
             try:
-                # On Python 3, comparing bytes/str is a TypeError, not a UnicodeError
+                # On Python 3, comparing bytes/str is a TypeError, not a
+                # UnicodeError
                 if isinstance(self._cmp_value, py3compat.bytestype):
                     return self._cmp_value > other.encode("utf-8")
                 return unicode(self._cmp_value) > other
@@ -585,7 +624,7 @@ class Literal(Identifier):
     def __ge__(self, other):
         if other is None:
             return False
-        if self==other:
+        if self == other:
             return True
         else:
             return self > other
@@ -610,28 +649,31 @@ class Literal(Identifier):
         >>> Literal('1', datatype=XSD.double) in a
         False
 
-        
-        "Called for the key object for dictionary operations, 
-        and by the built-in function hash(). Should return 
-        a 32-bit integer usable as a hash value for 
-        dictionary operations. The only required property 
-        is that objects which compare equal have the same 
-        hash value; it is advised to somehow mix together 
-        (e.g., using exclusive or) the hash values for the 
-        components of the object that also play a part in 
+
+        "Called for the key object for dictionary operations,
+        and by the built-in function hash(). Should return
+        a 32-bit integer usable as a hash value for
+        dictionary operations. The only required property
+        is that objects which compare equal have the same
+        hash value; it is advised to somehow mix together
+        (e.g., using exclusive or) the hash values for the
+        components of the object that also play a part in
         comparison of objects." -- 3.4.1 Basic customization (Python)
 
         "Two literals are equal if and only if all of the following hold:
-        * The strings of the two lexical forms compare equal, character by character.
+        * The strings of the two lexical forms compare equal, character by
+        character.
         * Either both or neither have language tags.
         * The language tags, if any, compare equal.
         * Either both or neither have datatype URIs.
-        * The two datatype URIs, if any, compare equal, character by character."
+        * The two datatype URIs, if any, compare equal, character by
+        character."
         -- 6.5.1 Literal Equality (RDF: Concepts and Abstract Syntax)
 
         """
-        
-        return Identifier.__hash__(self) ^ hash(self.language) ^ hash(self.datatype)
+
+        return Identifier.__hash__(self) \
+            ^ hash(self.language) ^ hash(self.datatype)
 
     @py3compat.format_doctest_out
     def __eq__(self, other):
@@ -639,14 +681,17 @@ class Literal(Identifier):
         >>> f = URIRef("foo")
         >>> f is None or f == ''
         False
-        >>> Literal("1", datatype=URIRef("foo")) == Literal("1", datatype=URIRef("foo"))
+        >>> Literal("1", datatype=URIRef("foo")) == Literal(
+        ...     "1", datatype=URIRef("foo"))
         True
-        >>> Literal("1", datatype=URIRef("foo")) == Literal("2", datatype=URIRef("foo"))
+        >>> Literal("1", datatype=URIRef("foo")) == Literal(
+        ...     "2", datatype=URIRef("foo"))
         False
         >>> Literal("1", datatype=URIRef("foo")) == "asdf"
         False
         >>> from rdflib.namespace import XSD
-        >>> Literal('2007-01-01', datatype=XSD.date) == Literal('2007-01-01', datatype=XSD.date)
+        >>> Literal('2007-01-01', datatype=XSD.date) == Literal(
+        ...     '2007-01-01', datatype=XSD.date)
         True
         >>> Literal('2007-01-01', datatype=XSD.date) == date(2007, 1, 1)
         True
@@ -756,6 +801,17 @@ class Literal(Identifier):
             >>> Literal(False)._literal_n3(use_plain=True)
             %(u)s'false'
 
+            >>> Literal(1.91)._literal_n3(use_plain=True)
+            %(u)s'1.91e+00'
+
+            Only limited precision available for floats:
+            >>> Literal(0.123456789)._literal_n3(use_plain=True)
+            %(u)s'1.234568e-01'
+
+            >>> Literal('0.123456789',
+            ...     datatype=XSD.decimal)._literal_n3(use_plain=True)
+            %(u)s'0.123456789'
+
         Using callback for datatype QNames::
 
             >>> Literal(1)._literal_n3(
@@ -765,18 +821,18 @@ class Literal(Identifier):
         '''
         if use_plain and self.datatype in _PLAIN_LITERAL_TYPES:
             try:
-                self.toPython() # check validity
-                # this is a bit of a mess - 
+                self.toPython()  # check validity
+                # this is a bit of a mess -
                 # in py >=2.6 the string.format function makes this easier
                 # we try to produce "pretty" output
-                if self.datatype == _XSD_DOUBLE: 
-                    return sub(".?0*e","e", u'%e' % float(self))
+                if self.datatype == _XSD_DOUBLE:
+                    return sub("\\.?0*e", "e", u'%e' % float(self))
                 elif self.datatype == _XSD_DECIMAL:
-                    return sub("0*$","0",u'%f' % float(self))
+                    return unicode(self._cmp_value)
                 else:
                     return u'%s' % self
             except ValueError:
-                pass # if it's in, we let it out?
+                pass  # if it's in, we let it out?
 
         encoded = self._quote_encode()
 
@@ -802,27 +858,29 @@ class Literal(Identifier):
     def _quote_encode(self):
         # This simpler encoding doesn't work; a newline gets encoded as "\\n",
         # which is ok in sourcecode, but we want "\n".
-        #encoded = self.encode('unicode-escape').replace(
+        # encoded = self.encode('unicode-escape').replace(
         #        '\\', '\\\\').replace('"','\\"')
-        #encoded = self.replace.replace('\\', '\\\\').replace('"','\\"')
+        # encoded = self.replace.replace('\\', '\\\\').replace('"','\\"')
 
         # NOTE: Could in theory chose quotes based on quotes appearing in the
         # string, i.e. '"' and "'", but N3/turtle doesn't allow "'"(?).
 
-        # which is nicer?
-        # if self.find("\"")!=-1 or self.find("'")!=-1 or self.find("\n")!=-1:
         if "\n" in self:
             # Triple quote this string.
             encoded = self.replace('\\', '\\\\')
             if '"""' in self:
                 # is this ok?
-                encoded = encoded.replace('"""','\\"\\"\\"')
-            if encoded.endswith('"'):
-                encoded = encoded[:-1] + "\\\""
-            return '"""%s"""' % encoded
+                encoded = encoded.replace('"""', '\\"\\"\\"')
+            if encoded[-1] == '"' and encoded[-2] != '\\':
+                encoded = encoded[:-1] + '\\' + '"'
+
+            return '"""%s"""' % encoded.replace('\r', '\\r')
         else:
-            return '"%s"' % self.replace('\n','\\n').replace('\\', '\\\\'
-                            ).replace('"', '\\"')
+            return '"%s"' % self.replace(
+                '\n', '\\n').replace(
+                    '\\', '\\\\').replace(
+                        '"', '\\"').replace(
+                            '\r', '\\r')
 
     if not py3compat.PY3:
         def __str__(self):
@@ -855,9 +913,9 @@ class Literal(Identifier):
     def _toCompareValue(self):
         try:
             rt = self.toPython()
-        except Exception, e:
-            _LOGGER.warning("could not convert %s to a Python datatype" % 
-                            repr(self))
+        except Exception:
+            _LOGGER.warning(
+                "could not convert %s to a Python datatype" % repr(self))
             rt = self
 
         if rt is self:
@@ -879,17 +937,16 @@ class Literal(Identifier):
         return d.hexdigest()
 
 
-
 _XSD_PFX = 'http://www.w3.org/2001/XMLSchema#'
 
-_XSD_FLOAT = URIRef(_XSD_PFX+'float')
-_XSD_DOUBLE = URIRef(_XSD_PFX+'double')
-_XSD_DECIMAL = URIRef(_XSD_PFX+'decimal')
+_XSD_FLOAT = URIRef(_XSD_PFX + 'float')
+_XSD_DOUBLE = URIRef(_XSD_PFX + 'double')
+_XSD_DECIMAL = URIRef(_XSD_PFX + 'decimal')
 
 
 _PLAIN_LITERAL_TYPES = (
-    URIRef(_XSD_PFX+'integer'),
-    URIRef(_XSD_PFX+'boolean'),
+    URIRef(_XSD_PFX + 'integer'),
+    URIRef(_XSD_PFX + 'boolean'),
     _XSD_DOUBLE,
     _XSD_DECIMAL,
 )
@@ -900,7 +957,7 @@ def _castPythonToLiteral(obj):
     Casts a python datatype to a tuple of the lexical value and a
     datatype URI (or None)
     """
-    for pType,(castFunc,dType) in _PythonToXSD:
+    for pType, (castFunc, dType) in _PythonToXSD:
         if isinstance(obj, pType):
             if castFunc:
                 return castFunc(obj), dType
@@ -908,81 +965,85 @@ def _castPythonToLiteral(obj):
                 return obj, dType
             else:
                 return obj, None
-    return obj, None # TODO: is this right for the fall through case?
+    return obj, None  # TODO: is this right for the fall through case?
 
 from decimal import Decimal
 
 # Mappings from Python types to XSD datatypes and back (burrowed from sparta)
 # datetime instances are also instances of date... so we need to order these.
 
-# SPARQL/Turtle/N3 has shortcuts for int, double, decimal 
+# SPARQL/Turtle/N3 has shortcuts for int, double, decimal
 # python has only float - to be in tune with sparql/n3/turtle
 # we default to XSD.double for float literals
 
 _PythonToXSD = [
     (basestring, (None, None)),
-    (float     , (None, URIRef(_XSD_PFX+'double'))),
-    (bool      , (lambda i:str(i).lower(), URIRef(_XSD_PFX+'boolean'))),
-    (int       , (None, URIRef(_XSD_PFX+'integer'))),
-    (long      , (None, URIRef(_XSD_PFX+'long'))),
-    (Decimal   , (None, URIRef(_XSD_PFX+'decimal'))),
-    (datetime  , (lambda i:i.isoformat(), URIRef(_XSD_PFX+'dateTime'))),
-    (date      , (lambda i:i.isoformat(), URIRef(_XSD_PFX+'date'))),
-    (time      , (lambda i:i.isoformat(), URIRef(_XSD_PFX+'time'))),
+    (float, (None, URIRef(_XSD_PFX + 'double'))),
+    (bool, (lambda i:str(i).lower(), URIRef(_XSD_PFX + 'boolean'))),
+    (int, (None, URIRef(_XSD_PFX + 'integer'))),
+    (long, (None, URIRef(_XSD_PFX + 'integer'))),
+    (Decimal, (None, URIRef(_XSD_PFX + 'decimal'))),
+    (datetime, (lambda i:i.isoformat(), URIRef(_XSD_PFX + 'dateTime'))),
+    (date, (lambda i:i.isoformat(), URIRef(_XSD_PFX + 'date'))),
+    (time, (lambda i:i.isoformat(), URIRef(_XSD_PFX + 'time'))),
 ]
 
 XSDToPython = {
-    URIRef(_XSD_PFX+'time')               : parse_time,
-    URIRef(_XSD_PFX+'date')               : parse_date,
-    URIRef(_XSD_PFX+'dateTime')           : parse_datetime,
-    URIRef(_XSD_PFX+'string')             : None,
-    URIRef(_XSD_PFX+'normalizedString')   : None,
-    URIRef(_XSD_PFX+'token')              : None,
-    URIRef(_XSD_PFX+'language')           : None,
-    URIRef(_XSD_PFX+'boolean')            : lambda i:i.lower() in ['1','true'],
-    URIRef(_XSD_PFX+'decimal')            : Decimal,
-    URIRef(_XSD_PFX+'integer')            : long,
-    URIRef(_XSD_PFX+'nonPositiveInteger') : int,
-    URIRef(_XSD_PFX+'long')               : long,
-    URIRef(_XSD_PFX+'nonNegativeInteger') : int,
-    URIRef(_XSD_PFX+'negativeInteger')    : int,
-    URIRef(_XSD_PFX+'int')                : long,
-    URIRef(_XSD_PFX+'unsignedLong')       : long,
-    URIRef(_XSD_PFX+'positiveInteger')    : int,
-    URIRef(_XSD_PFX+'short')              : int,
-    URIRef(_XSD_PFX+'unsignedInt')        : long,
-    URIRef(_XSD_PFX+'byte')               : int,
-    URIRef(_XSD_PFX+'unsignedShort')      : int,
-    URIRef(_XSD_PFX+'unsignedByte')       : int,
-    URIRef(_XSD_PFX+'float')              : float,
-    URIRef(_XSD_PFX+'double')             : float,
-    URIRef(_XSD_PFX+'base64Binary')       : lambda s: base64.b64decode(py3compat.b(s)),
-    URIRef(_XSD_PFX+'anyURI')             : None,
+    URIRef(_XSD_PFX + 'time'): parse_time,
+    URIRef(_XSD_PFX + 'date'): parse_date,
+    URIRef(_XSD_PFX + 'dateTime'): parse_datetime,
+    URIRef(_XSD_PFX + 'string'): None,
+    URIRef(_XSD_PFX + 'normalizedString'): None,
+    URIRef(_XSD_PFX + 'token'): None,
+    URIRef(_XSD_PFX + 'language'): None,
+    URIRef(_XSD_PFX + 'boolean'): lambda i: i.lower() in ['1', 'true'],
+    URIRef(_XSD_PFX + 'decimal'): Decimal,
+    URIRef(_XSD_PFX + 'integer'): long,
+    URIRef(_XSD_PFX + 'nonPositiveInteger'): int,
+    URIRef(_XSD_PFX + 'long'): long,
+    URIRef(_XSD_PFX + 'nonNegativeInteger'): int,
+    URIRef(_XSD_PFX + 'negativeInteger'): int,
+    URIRef(_XSD_PFX + 'int'): long,
+    URIRef(_XSD_PFX + 'unsignedLong'): long,
+    URIRef(_XSD_PFX + 'positiveInteger'): int,
+    URIRef(_XSD_PFX + 'short'): int,
+    URIRef(_XSD_PFX + 'unsignedInt'): long,
+    URIRef(_XSD_PFX + 'byte'): int,
+    URIRef(_XSD_PFX + 'unsignedShort'): int,
+    URIRef(_XSD_PFX + 'unsignedByte'): int,
+    URIRef(_XSD_PFX + 'float'): float,
+    URIRef(_XSD_PFX + 'double'): float,
+    URIRef(
+        _XSD_PFX + 'base64Binary'): lambda s: base64.b64decode(py3compat.b(s)),
+    URIRef(_XSD_PFX + 'anyURI'): None,
 }
 
 _toPythonMapping = {}
 _toPythonMapping.update(XSDToPython)
 
+
 def bind(datatype, conversion_function):
     """
     bind a datatype to a function for converting it into a Python
     instance.
     """
     if datatype in _toPythonMapping:
-        _LOGGER.warning("datatype '%s' was already bound. Rebinding." % 
+        _LOGGER.warning("datatype '%s' was already bound. Rebinding." %
                         datatype)
     _toPythonMapping[datatype] = conversion_function
 
 
-
 class Variable(Identifier):
     """
     """
     __slots__ = ()
+
     def __new__(cls, value):
-        if len(value)==0: raise Exception("Attempted to create variable with empty string as name!")
-        if value[0]=='?':
-            value=value[1:]
+        if len(value) == 0:
+            raise Exception(
+                "Attempted to create variable with empty string as name!")
+        if value[0] == '?':
+            value = value[1:]
         return unicode.__new__(cls, value)
 
     def __repr__(self):
@@ -1012,6 +1073,10 @@ class Variable(Identifier):
 class Statement(Node, tuple):
 
     def __new__(cls, (subject, predicate, object), context):
+        warnings.warn(
+            "Class Statement is deprecated, and will be removed in " +
+            "the future. If you use this please let rdflib-dev know!",
+            category=DeprecationWarning, stacklevel=2)
         return tuple.__new__(cls, ((subject, predicate, object), context))
 
     def __reduce__(self):
@@ -1024,5 +1089,3 @@ class Statement(Node, tuple):
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
-
-
